package data_manage

import (
	"eta/eta_api/models"
	"eta/eta_api/models/data_manage"
	"eta/eta_api/utils"
	"fmt"
	"github.com/rdlucklib/rdluck_tools/paging"
	"github.com/tealeg/xlsx"
	"os"
	"path/filepath"
	"strings"
	"time"
)

// FenweiClassify
// @Title 汾渭数据分类
// @Description 汾渭数据分类接口
// @Success 200 {object} data_manage.BaseFromFenweiClassifyItem
// @router /fenwei/classify [get]
func (this *EdbInfoController) FenweiClassify() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()
	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	classifies, e := data_manage.GetBaseFromFenweiClassify()
	if e != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取汾渭数据分类失败, Err: " + e.Error()
		return
	}

	classifyOb := new(data_manage.BaseFromFenweiClassify)
	resp := make([]*data_manage.BaseFromFenweiClassifyItem, 0)
	parentMap := make(map[int][]*data_manage.BaseFromFenweiClassifyItem)
	for _, v := range classifies {
		t := classifyOb.Format2Item(v)
		if v.ParentId == 0 {
			resp = append(resp, t)
		}
		if v.ParentId > 0 {
			if parentMap[v.ParentId] == nil {
				parentMap[v.ParentId] = make([]*data_manage.BaseFromFenweiClassifyItem, 0)
			}
			parentMap[v.ParentId] = append(parentMap[v.ParentId], t)
		}
	}
	for _, v := range resp {
		v.Child = parentMap[v.ClassifyId]
	}

	br.Data = resp
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
}

// FenweiIndexData
// @Title 获取汾渭数据
// @Description 获取汾渭数据接口
// @Param   PageSize   query   int  true       "每页数据条数"
// @Param   CurrentIndex   query   int  true       "当前页页码，从1开始"
// @Param   ClassifyId   query   string  true       "分类id"
// @Success 200 {object} data_manage.LzFrequency
// @router /fenwei/index/data [get]
func (this *EdbInfoController) FenweiIndexData() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()
	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	pageSize, _ := this.GetInt("PageSize")
	currentIndex, _ := this.GetInt("CurrentIndex")
	var startSize int

	if pageSize <= 0 {
		pageSize = utils.PageSize20
	}
	if currentIndex <= 0 {
		currentIndex = 1
	}
	startSize = utils.StartIndex(currentIndex, pageSize)

	classifyId, _ := this.GetInt("ClassifyId")
	if classifyId < 0 {
		br.Msg = "请选择分类"
		br.ErrMsg = "请选择分类"
		return
	}

	// 获取指标
	var condition string
	var pars []interface{}
	if classifyId >= 0 {
		condition += ` AND classify_id=? `
		pars = append(pars, classifyId)
	}

	indexes, err := data_manage.GetFenweiIndex(condition, pars)
	if err != nil {
		br.Msg = "获取数据失败"
		br.ErrMsg = "获取数据失败,Err:" + err.Error()
		return
	}

	indexCodes := make([]string, 0)
	for _, v := range indexes {
		indexCodes = append(indexCodes, v.IndexCode)
	}
	indexCounts, e := data_manage.GetFenweiIndexDataCountGroup(indexCodes)
	if e != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取指标数据总量失败, Err:" + err.Error()
		return
	}
	countMap := make(map[string]int)
	for _, v := range indexCounts {
		countMap[v.IndexCode] = v.Count
	}

	resultList := make([]*data_manage.BaseFromFenweiIndexList, 0)
	for _, v := range indexes {
		product := new(data_manage.BaseFromFenweiIndexList)
		product.FenweiIndexId = v.FenweiIndexId
		product.ClassifyId = v.ClassifyId
		product.Unit = v.Unit
		product.IndexCode = v.IndexCode
		product.IndexName = v.IndexName
		product.Frequency = v.Frequency
		product.CreateTime = v.CreateTime
		product.ModifyTime = v.ModifyTime

		total := countMap[v.IndexCode]
		page := paging.GetPaging(currentIndex, pageSize, total)
		dataList, e := data_manage.GetFenweiIndexData(v.IndexCode, startSize, pageSize)
		if e != nil {
			br.Msg = "获取数据失败"
			br.ErrMsg = "获取指标数据失败,Err:" + e.Error()
			return
		}
		if dataList == nil {
			dataList = make([]*data_manage.BaseFromFenweiData, 0)
		}
		product.DataList = dataList
		product.Paging = page
		resultList = append(resultList, product)
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resultList
}

// FenweiSearchList
// @Title Fenwei模糊搜索
// @Description Fenwei模糊搜索
// @Param   Keyword   query   string  ture       "关键字搜索"
// @Success 200 {object} data_manage.BaseFromFenweiIndexSearchItem
// @router /fenwei/search_list [get]
func (this *EdbInfoController) FenweiSearchList() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()
	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	list := make([]*data_manage.BaseFromFenweiIndexSearchItem, 0)
	var err error
	// 关键字
	keyword := this.GetString("Keyword")
	if keyword != "" {
		keyWordArr := strings.Split(keyword, " ")

		if len(keyWordArr) > 0 {
			condition := ""
			for _, v := range keyWordArr {
				condition += ` AND CONCAT(index_name,index_code) LIKE '%` + v + `%'`
			}
			list, err = data_manage.GetFenweiItemList(condition)
			if err != nil {
				br.ErrMsg = "获取失败，Err:" + err.Error()
				br.Msg = "获取失败"
				return
			}
		}
	} else {
		list, err = data_manage.GetFenweiItemList("")
		if err != nil {
			br.ErrMsg = "获取失败，Err:" + err.Error()
			br.Msg = "获取失败"
			return
		}
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = list
}

// FenweiSingleData
// @Title 获取Fenwei数据
// @Description 获取Fenwei单条数据接口
// @Param   IndexCode   query   string  true       "指标唯一编码"
// @Success 200 {object} models.BaseResponse
// @router /fenwei/single_data [get]
func (this *EdbInfoController) FenweiSingleData() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()
	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}
	indexCode := this.GetString("IndexCode")
	indexInfo, err := data_manage.GetBaseFromFenweiIndexByIndexCode(indexCode)
	if err != nil {
		br.Msg = "获取指标信息失败"
		br.ErrMsg = "获取指标信息失败,Err:" + err.Error()
		return
	}
	dataTmpList, err := data_manage.GetFenweiIndexDataByCode(indexCode)
	if err != nil {
		br.Msg = "获取数据失败"
		br.ErrMsg = "获取数据失败,Err:" + err.Error()
		return
	}
	var ret data_manage.FenweiSingleDataResp
	var dataList []*data_manage.FenweiSingleData

	ret.ClassifyId = indexInfo.ClassifyId
	ret.FenweiIndexId = indexInfo.FenweiIndexId
	ret.IndexCode = indexInfo.IndexCode
	ret.IndexName = indexInfo.IndexName
	ret.Frequency = indexInfo.Frequency
	ret.CreateTime = indexInfo.CreateTime.Format(utils.FormatDateTime)
	ret.ModifyTime = indexInfo.ModifyTime.Format(utils.FormatDateTime)
	ret.Unit = indexInfo.Unit
	for _, v := range dataTmpList {
		tmp := &data_manage.FenweiSingleData{
			Value:    v.Value,
			DataTime: v.DataTime,
		}
		dataList = append(dataList, tmp)
	}
	ret.Data = dataList

	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = ret
}

// ExportFenweiList
// @Title 导出Fenwei数据
// @Description 导出Fenwei数据
// @Param   IndexName   query   string  false       "名称关键词"
// @Param   IndexCode   query   string  false       "指标唯一编码"
// @Param   ClassifyId   query   string  true       "分类"
// @Success 200  导出成功
// @router /fenwei/export [get]
func (this *EdbInfoController) ExportFenweiList() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请重新登录"
		return
	}

	indexCode := this.GetString("IndexCode") //指标唯一编码
	classifyId, _ := this.GetInt("ClassifyId")
	//secNameList := make([]*models.EdbdataExportList, 0)

	dir, _ := os.Executable()
	exPath := filepath.Dir(dir)

	downLoadnFilePath := exPath + "/" + time.Now().Format(utils.FormatDateTimeUnSpace) + ".xlsx"
	xlsxFile := xlsx.NewFile()

	var pars []interface{}
	condition := ""
	if classifyId > 0 {
		//获取指标
		condition += " AND classify_id=?"
		pars = append(pars, classifyId)
	}
	if indexCode != "" {
		//获取指标
		condition += " AND index_code=?"
		pars = append(pars, indexCode)
	}
	indexList, err := data_manage.GetFenweiIndex(condition, pars)
	if err != nil {
		fmt.Println("获取数据失败，Err:" + err.Error())
		return
	}
	if len(indexList) <= 0 {
		fmt.Println("indexList 为空")
		br.Ret = 200
		br.Success = true
		br.Msg = "success"
		return
	}
	sheetNew := new(xlsx.Sheet)
	// todo 分类名称
	sheetNew, err = xlsxFile.AddSheet("汾渭数据")
	//sheetNew.SetColWidth()
	//获取指标数据
	windRow := sheetNew.AddRow()
	secNameRow := sheetNew.AddRow()
	indexCodeRow := sheetNew.AddRow()
	frequencyRow := sheetNew.AddRow()
	unitRow := sheetNew.AddRow()
	lastModifyDateRow := sheetNew.AddRow()
	//获取分类下指标最大数据量
	var dataMax int
	setRowIndex := 6
	indexCodeList := make([]string, 0)
	for _, v := range indexList {
		indexCodeList = append(indexCodeList, v.IndexCode)
	}
	dataListMap := make(map[string][]*data_manage.BaseFromFenweiData)
	if len(indexList) > 0 {
		allDataList, e := data_manage.GetFenweiIndexDataByCodes(indexCodeList)
		if e != nil {
			br.Msg = "获取数据失败"
			br.ErrMsg = "获取数据失败,Err:" + e.Error()
			return
		}
		for _, v := range allDataList {
			dataListMap[v.IndexCode] = append(dataListMap[v.IndexCode], v)
		}
		for _, v := range dataListMap {
			if len(v) > dataMax {
				dataMax = len(v)
			}
		}
	}

	for k, sv := range indexList {
		//获取数据
		dataList, ok := dataListMap[sv.IndexCode]
		if !ok {
			continue
		}
		if len(dataList) > 0 {
			windRow.AddCell().SetValue("汾渭数据")
			secNameRow.AddCell().SetValue("指标名称")
			indexCodeRow.AddCell().SetValue("指标ID")
			frequencyRow.AddCell().SetValue("频率")
			unitRow.AddCell().SetValue("单位")
			lastModifyDateRow.AddCell().SetValue("更新时间")

			secNameRow.AddCell().SetValue(sv.IndexName)
			indexCodeRow.AddCell().SetValue(sv.IndexCode)
			frequencyRow.AddCell().SetValue(sv.Frequency)

			unitRow.AddCell().SetValue(sv.Unit)
			lastModifyDateRow.AddCell().SetValue(sv.ModifyTime)

			windRow.AddCell()
			windRow.AddCell()
			secNameRow.AddCell()
			indexCodeRow.AddCell()
			frequencyRow.AddCell()
			unitRow.AddCell()
			lastModifyDateRow.AddCell()
			min := k * 3
			sheetNew.SetColWidth(min, min, 15)

			if len(dataList) <= 0 {
				for n := 0; n < dataMax; n++ {
					rowIndex := setRowIndex + n
					row := sheetNew.Row(rowIndex)
					row.AddCell()
					row.AddCell()
					row.AddCell()
				}
			} else {
				endRowIndex := 0
				for rk, dv := range dataList {
					rowIndex := setRowIndex + rk
					row := sheetNew.Row(rowIndex)
					displayDate, _ := time.Parse(utils.FormatDate, dv.DataTime)
					displayDateCell := row.AddCell()
					style := new(xlsx.Style)
					style.ApplyAlignment = true
					style.Alignment.WrapText = true
					displayDateCell.SetStyle(style)
					displayDateCell.SetDate(displayDate)

					row.AddCell().SetValue(dv.Value)
					row.AddCell()
					endRowIndex = rowIndex
				}
				if len(dataList) < dataMax {
					dataLen := dataMax - len(dataList)
					for n := 0; n < dataLen; n++ {
						rowIndex := (endRowIndex + 1) + n
						row := sheetNew.Row(rowIndex)
						row.AddCell()
						row.AddCell()
						row.AddCell()
					}
				}
			}
		}
	}

	err = xlsxFile.Save(downLoadnFilePath)
	if err != nil {
		//有指标无数据时先导出一遍空表
		sheet, err := xlsxFile.AddSheet("无数据")
		if err != nil {
			br.Msg = "新增Sheet失败"
			br.ErrMsg = "新增Sheet失败,Err:" + err.Error()
			return
		}
		rowSecName := sheet.AddRow()
		celSecName := rowSecName.AddCell()
		celSecName.SetValue("")
		err = xlsxFile.Save(downLoadnFilePath)
		if err != nil {
			br.Msg = "保存文件失败"
			br.ErrMsg = "保存文件失败"
			return
		}
	}
	fileName := `汾渭数据`
	if len(indexList) > 0 {
		fileName = indexList[0].IndexName
	}
	fileName += time.Now().Format("06.01.02") + `.xlsx` //文件名称
	this.Ctx.Output.Download(downLoadnFilePath, fileName)
	defer func() {
		os.Remove(downLoadnFilePath)
	}()
	br.Ret = 200
	br.Success = true
	br.Msg = "success"

}
